import logging
import datetime

from pylons import request, response, session, tmpl_context as c, url
from pylons.controllers.util import abort, redirect

import webhelpers.paginate as paginate

from mapwocgui.lib.base import BaseController, render

from mapwocgui.model import Session
from mapwoc.lib.result import Visit, Result, ResultType
from mapwoc.lib.scanconfiguration import ScanRun
from mapwoc.lib.url import URL
from paste.fileapp import FileApp
from os import path

import mapwoc.lib.model.result
import mapwocgui.model.masterproxy as master

log = logging.getLogger(__name__)

class ResultController(BaseController):

    def index(self):
        visits = Session.query(Visit)
        c.visit_paginator = paginate.Page(visits,
                                 page=int(request.params.get('page', 1)),
                                 items_per_page=30,)
        c.resulttypes = Session.query(ResultType).all()
#        c.resulttype_names = [rt.type_name for rt in c.resulttypes]
        c.dtnow = datetime.datetime.now()
        c.scanruns = Session.query(ScanRun).all()
        return render('/result/index_result.mako')

    def by_type(self, id):
        query = Session.query(Result).filter_by(result_type_id=id)
        c.resulttypes = Session.query(ResultType).all()
        c.result_type_filter_id = id
        c.result_paginator = paginate.Page(query,
                                 page=int(request.params.get('page', 1)),
                                 items_per_page=30,)
        return render('/result/index_result.mako')

    def by_scanrun(self, id):
        pass

    def by_url(self, id):
        c.results = Session.query(Result).all()
        c.resulttypes = Session.query(ResultType).all()
        return render('/result/index_result.mako')

    def view_visit(self, id):
        c.visit = Session.query(Visit).filter_by(id=id).first()
        if c.visit is None:
            abort(404)
        c.scanrun = c.visit.scan_run
        c.sc = c.scanrun.scanconfiguration
        c.types_found = []
        # TODO: Localize
        c.firstDate = c.visit.results[0].date.strftime("%d.%m.%Y - %H:%M:%S")
        for result in c.visit.results:
            if not result.result_type in c.types_found:
                c.types_found.append(result.result_type)
        c.highest_severity = max([type.severity for type in c.types_found])
        c.hc = master.get_hc(c.visit.hcsid)

        return render('/result/view_visit.mako')

    def send_dump(self, id):
        pass

    def send_tarball(self, id):
        """ Sends the result tarball """
        visit = Session.query(Visit).filter_by(id=id).first()
        if visit is None:
            abort(404)

        if not visit.filepath:
            abort(404)
        user_filename = "Result_%s.tar.gz" % id
        file_size = path.getsize(visit.filepath)

        headers = [('Content-Disposition', 'attachment; filename=\"' + user_filename + '\"'),
                   ('Content-Type', 'text/plain'),
                   ('Content-Length', str(file_size))]

        fapp = FileApp(visit.filepath, headers=headers)
        return fapp(request.environ, self.start_response)

    def view_result(self, id):
        c.result = Session.query(Result).filter_by(id=id).first()
        if c.result is None:
            abort(404)
        c.visit =  c.result.visit # Session.query(Visit).filter(Visit.results.contains(c.result)).first()
        c.hc = master.get_hc(c.visit.hcsid)
        c.sc = c.visit.scan_run
        return render('/result/view_result.mako')

    def view_dummy(self):
        return render('/result/view_result.mako')
